%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CZS cv. 10
% vztah komplexni obalky (CE) a pasmoveho signalu (BP)

clear;

%% parametry signalu
%M = 2; %pocet stavu modulace
%M = 4; %pocet stavu modulace
M = 8; %pocet stavu modulace
A = 1; %amplituda dig. modulace
fsa = 55e3; %vzorkovaci kmitocet
Tsa = 1/fsa;
fs = 1e3; %symbolova rychlost
Ts = 1/fs;
Ns = 100; %pocet symbolu
fo = 10e3; %kmitocet nosne

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cast -- A1

%% generovani stejne pravdepodobnych nezavislych symbolu
data = fix(M*(rand(1,Ns) )); % cilsa 0,1,... M-1
symbl = exp(1j*2*pi/M*data); %komplexni symboly

%% genrovani komplexni obalky -- prevzorkovani symbl() na fsa
N = fix(Ns*Ts/Tsa); %pocet vzorku signalu
for i = 0:N-1,
  s_CE(i+1) = A*symbl( fix(i*Tsa/Ts) + 1 );
end; 

%% zobrazeni
figure(1);clf;
subplot(311);
plot( (0:N-1)*Tsa, real(s_CE), 'b');
hold on;
plot( (0:N-1)*Tsa, imag(s_CE), 'g');
xlabel('t [s]');
legend('In-phase','Quadrature');
title('CE without noise');

subplot(312);
plot(real(s_CE), imag(s_CE), 'x:' );
axis([-1.2,1.2,-1.2,1.2]);
xlabel('I(t)'); ylabel('Q(t)');
title('CE in complex plain');

subplot(313);
[PP, ww] = pwelch(s_CE,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
grid on;
xlabel('f [Hz]');
title('PSD of CE without noise [dB/Hz]');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cast -- A2

%% pridani komplexniho AWGN
P_No = 53; %[dB-Hz]
%P_No = 100; %[dB-Hz]

% No -- PSD soufazove nebo kvadraturni slozky
% vykon komplexni obalky je A^2, nebo lze ziskat i vypoctem 1/N*s_CE*s_CE'
No = A^2 / (10^(P_No/10));

% jaky ma byt vykon soufazove nebo kvadraturni slozky, kdyz kazda ma mit
% oboustrannou PSD = No ?
% Pn_I = Pn_Q = fsa/2 * No * 2, tedy fsa * No
n_CE = sqrt( fsa*No )*( randn(1,N) +1j*randn(1,N) );

% overte, ze vykon n_CE je 2*fsa*No
%var(n_CE)
%2*fsa*No

%% navrh filtru DP -- pouzijem ho jak pri priprave komplexni obalky (viz
% poznamka o nulovosti spektra pro f < -fo tak i pri BP->CE pomoci
% filtracni metody
b=firpm(40, [0 3e3 4e3 fsa/2]*2/fsa, [1 1 0 0]);
%sn_CE = s_CE + n_CE;
sn_CE = filter(b,1, s_CE + n_CE);


%% zobrazeni
figure(2);clf;
subplot(311);
plot( (0:N-1)*Tsa, real(sn_CE), 'b');
hold on;
plot( (0:N-1)*Tsa, imag(sn_CE), 'g');
xlabel('t [s]');
legend('In-phase','Quadrature');
title('CE with AWGN');

subplot(312);
plot(real(sn_CE), imag(sn_CE), 'x:' );
axis([-2.5,2.5,-2.5,2.5]);
xlabel('I(t)'); ylabel('Q(t)');
title('CE in complex plain');

subplot(313);
[PP, ww] = pwelch(sn_CE,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
grid on;
xlabel('f [Hz]');
title('PSD of CE with AWGN [dB/Hz]');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CAST -- B

%% prevod CE -> BP
% bud takto
%sn_BP = real( sn_CE.*exp(1j*2*pi*fo*(0:N-1)*Tsa) );
% nebo alternativne
sn_BP = real(sn_CE).*cos( 2*pi*fo*(0:N-1)*Tsa ) - ...
        imag(sn_CE).*sin( 2*pi*fo*(0:N-1)*Tsa );

%% zobrazeni
figure(3);clf;
subplot(211);
plot( (0:N-1)*Tsa, sn_BP, 'b');
xlabel('t [s]');
title('BP in time domain');

subplot(212);
[PP, ww] = pwelch(sn_BP,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
grid on;
xlabel('f [Hz]');
title('PSD of BP');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CAST -- C

%% prevod BP -> CE filtracni metodou

sn_I2 = filter(b,1,  2*sn_BP.*cos( 2*pi*fo*(0:N-1)*Tsa ) );
sn_Q2 = filter(b,1, -2*sn_BP.*sin( 2*pi*fo*(0:N-1)*Tsa ) );

sn_CE2 = sn_I2 + 1j*sn_Q2;

%% zobrazeni
figure(4);clf;
subplot(311);
plot( (0:N-1)*Tsa, real(sn_CE2), 'b');
hold on;
plot( (0:N-1)*Tsa, imag(sn_CE2), 'g');
legend('In-phase','Quadrature');
title('final CE');

subplot(312);
plot(real(sn_CE2), imag(sn_CE2), 'x:' );
axis([-2.5,2.5,-2.5,2.5]);
xlabel('I(t)'); ylabel('Q(t)');
title('final CE in complex plain');

subplot(313);
[PP, ww] = pwelch(sn_CE2,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
grid on;
xlabel('f [Hz]');
title('PSD of final CE [dB/Hz]');
