%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CZS cv. 10
% vztah komplexni obalky (CE) a pasmoveho signalu (BP)

clear;

%% parametry signalu
% pocet stavu modulace, postupne zkuste 2,4,8
M = 8;
% vykon pasmoveho signalu bez sumu (vykon komplexni obalky je 2*P)
P = 1;
% pomer vykon sigbalu ku spektralni vykonove hustote (v [dB-Hz])
P_No_dB = 53;
% vzorkovaci kmitoce / perioda
fsa = 55e3;
    Tsa = 1/fsa;
% symbolova rychlost, doba trvani symbolu
fs = 1e3;
    Ts = 1/fs;
% pocet symbolu, se kterymi budeme pracovat
Ns = 100;
    N = fix(Ns*Ts/Tsa);
% kmitocet nosne: sterd pasma pasmoveho signali
fo = 10e3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cast -- A1

%% generovani stejne pravdepodobnych nezavislych symbolu
data = fix(M*(rand(1,Ns) )); % cilsa 0,1,... M-1
symbl = exp(1j*2*pi/M*data); %komplexni symboly

%% genrovani komplexni obalky -- prevzorkovani symbl() na fsa
% varianta s for()
%for i = 0:N-1,
%  s_CE(i+1) = sqrt(2*P)*symbl( fix(i*Tsa/Ts) + 1 );
%end; 

% nebo lepe bez for() (ale mozna mene nazorne)
s_CE = sqrt(2*P)*symbl(  fix( (0:N-1)*Tsa/Ts ) + 1  );

%% zobrazeni
figure(1);clf;
subplot(311);
plot( (0:N-1)*Tsa, real(s_CE), 'b');
hold on;
plot( (0:N-1)*Tsa, imag(s_CE), 'g');
xlabel('t [s]');
legend('In-phase','Quadrature');
title('CE without noise');

subplot(312);
plot(real(s_CE), imag(s_CE), 'x:' );
axis([-1.2,1.2,-1.2,1.2]);
xlabel('I(t)'); ylabel('Q(t)');
title('CE in complex plain');

subplot(313);
[PP, ww] = pwelch(s_CE,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
grid on;
set(gca,'YLim',[-70,-20]);
xlabel('f [Hz]');
title('PSD of CE without noise [dB/Hz]');

%% pro kontrolu, vypocet vykonu s_CE
P_CE_time = s_CE*s_CE'/N;
P_CE_freq = sum(PP)*fsa/length(PP);
fprintf(1, 'power of CE: %2.2f, %2.2f\n', P_CE_time, P_CE_freq);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cast -- A2

% No -- PSD soufazove nebo kvadraturni slozky
No = P / (10^(P_No_dB/10));

% jaky ma byt vykon soufazove nebo kvadraturni slozky, kdyz kazda ma mit
% oboustrannou PSD = No ?
% Pn_I = Pn_Q = fsa/2 * No * 2, tedy fsa * No
n_CE = sqrt( fsa*No )*( randn(1,N) +1j*randn(1,N) );

% overte, ze vykon n_CE je 2*fsa*No
fprintf(1, 'Power of inphase/quadrature noise component: %2.2f, %2.2f\n',...
    var(n_CE), 2*fsa*No );

%% navrh filtru DP
% filtr meni vykon sumu, ale nemeni PSD ve zpracovavanem pasmu
fco = 15e3; % filter cutoff frequency
b1=firpm(40, [0 fco fco+1e3 fsa/2]*2/fsa, [1 1 0 0]);

% varianta: komplexni obalka s filtrovanym/nefiltrovanym sumem
sn_CE = s_CE + n_CE;
%sn_CE = filter(b1,1, s_CE + n_CE);


%% zobrazeni
figure(2);clf;
subplot(311);
plot( (0:N-1)*Tsa, real(sn_CE), 'b');
hold on;
plot( (0:N-1)*Tsa, imag(sn_CE), 'g');
xlabel('t [s]');
legend('In-phase','Quadrature');
title('CE with AWGN');

subplot(312);
plot(real(sn_CE), imag(sn_CE), 'x:' );
axis([-2.5,2.5,-2.5,2.5]);
xlabel('I(t)'); ylabel('Q(t)');
title('CE in complex plain');

subplot(313);
[PP, ww] = pwelch(sn_CE,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
set(gca,'YLim',[-70,-20]);
grid on; hold on;
plot(ww - ww(fix(length(ww)/2)+1), ones(length(PP),1)*10*log10(2*No),'r-');
xlabel('f [Hz]');
title('PSD of CE with AWGN [dB/Hz]');
legend('signal PSD', 'AWGN PSD: 2*No');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CAST -- B

% POZOR: pred prevodem CE->BP je potreba zajistit, aby spektrum sn_CE
% bylo nulove pro |f| > fo. Bud lze pouzit dostatecne uzky filtr
% z predchoziho kroku, nebo znovu filtrovat.
% Nedodrzeni podminky: nesedi predpoklady o PSD sumu 2*No u CE a No/2
% u BP, viz PSD v okolu nuly.

b2=firpm(40, [0 fo-1e3 fo fsa/2]*2/fsa, [1 1 0 0]);
sn_CE = filter(b2, 1, sn_CE);

%% prevod CE -> BP
% bud takto
%sn_BP = real( sn_CE.*exp(1j*2*pi*fo*(0:N-1)*Tsa) );
% nebo alternativne
sn_BP = real(sn_CE).*cos( 2*pi*fo*(0:N-1)*Tsa ) - ...
        imag(sn_CE).*sin( 2*pi*fo*(0:N-1)*Tsa );


%% uloznei BP do souboru:
% v ramci domaciho cviceni lze pak pouzit k prevodu BP->CP jinou 
% metodou (pomoci Hilbertova transformatoru, prevod ve frekvencni
% oblasti) 

%% zobrazeni
figure(3);clf;
subplot(211);
plot( (0:N-1)*Tsa, sn_BP, 'b');
xlabel('t [s]');
title('BP in time domain');

subplot(212);
[PP, ww] = pwelch(sn_BP,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
set(gca,'YLim',[-70,-30]);
grid on; hold on;
plot(ww - ww(fix(length(ww)/2)+1), ones(length(PP),1)*10*log10(No/2),'r-');
xlabel('f [Hz]');
title('PSD of BP');
legend('signal PSD', 'AWGN PSD: No/2');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CAST -- C

%% prevod BP -> CE filtracni metodou

sn_I2 = filter(b2,1,  2*sn_BP.*cos( 2*pi*fo*(0:N-1)*Tsa ) );
sn_Q2 = filter(b2,1, -2*sn_BP.*sin( 2*pi*fo*(0:N-1)*Tsa ) );

sn_CE2 = sn_I2 + 1j*sn_Q2;

%% nebo

sn_CE_moved = sn_BP.*exp(-1j*2*pi*fo*(0:N-1)*Tsa) ;
sn_CE3 = filter(b2,1,sn_CE_moved) ;

%% zobrazeni
figure(4);clf;
subplot(311);
plot( (0:N-1)*Tsa, real(sn_CE2), 'b');
hold on;
plot( (0:N-1)*Tsa, imag(sn_CE2), 'g');
legend('In-phase','Quadrature');
title('final CE');

subplot(312);
plot(real(sn_CE2), imag(sn_CE2), 'x:' );
axis([-2.5,2.5,-2.5,2.5]);
xlabel('I(t)'); ylabel('Q(t)');
title('final CE in complex plain');

subplot(313);
[PP, ww] = pwelch(sn_CE2,5*fix(Ts/Tsa),[],[],fsa,'twosided');
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
grid on;
xlabel('f [Hz]');
title('PSD of final CE [dB/Hz]');
set(gca,'YLim',[-70,-20]);
grid on; hold on;
plot(ww - ww(fix(length(ww)/2)+1), ones(length(PP),1)*10*log10(2*No),'r-');
xlabel('f [Hz]');
legend('signal PSD', 'AWGN PSD: 2*No');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(5)

subplot(211);
semilogy(abs(freqz(b1,1,1000)))

subplot(212)
semilogy(abs(freqz(b2,1,1000)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(6)
subplot(311)
[PP, ww] = pwelch(sn_BP,5*fix(Ts/Tsa),[],[],fsa,'twosided')
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
subplot(312)
[PP, ww] = pwelch(2*sn_BP.*cos( 2*pi*fo*(0:N-1)*Tsa ) ,5*fix(Ts/Tsa),[],[],fsa,'twosided')
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
subplot(313)
[PP, ww] = pwelch(-2*sn_BP.*sin( 2*pi*fo*(0:N-1)*Tsa ) ,5*fix(Ts/Tsa),[],[],fsa,'twosided')
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(7)
subplot(311)
[PP, ww] = pwelch(sn_BP,5*fix(Ts/Tsa),[],[],fsa,'twosided')
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
subplot(312)
[PP, ww] = pwelch(sn_CE_moved,5*fix(Ts/Tsa),[],[],fsa,'twosided')
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 
subplot(313)
[PP, ww] = pwelch(sn_CE3,5*fix(Ts/Tsa),[],[],fsa,'twosided')
plot(ww - ww(fix(length(ww)/2)+1), 10*log10( fftshift(PP) ) ); 

