% ML estimation of distance measurement for GPS location-based services
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if exist('/etc/init.d'),
    % For linux
    addpath /home/pollak/www/html/vyu/b2m31czs/m
    addpath /home/pollak/www/html/vyu/b2m31czs/signaly
else
    % For windows
    if exist('H:\VYUKA\CZS')
        addpath H:\VYUKA\CZS\m; 
        addpath H:\VYUKA\CZS\signaly; 
    else
        addpath C:\usr\pollak\www\html\vyu\b2m31czs\m
        addpath C:\usr\pollak\www\html\vyu\b2m31czs\signaly
    end
end

clear;

% PRN sequence (Pseudo-Random Noise and his chip rate (sampling frequency)
prn = load('prn1.txt');
fch = 1.023e6;
Tch = 1/fch;

% Measured signal and his sampling frequency 
% signal = load('sig1.gps');
% signal = load('sig2.gps');
% signal = load('sig3.gps');
% signal = load('sig4.gps');
signal = load('sig5.gps');
% signal = load('sig6.gps');
fsa = 65e6;
Tsa = 1/fsa;

N = length(signal);

% Creation of trasmitted signal, i.e. resampled replica of PRN sequence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
replika = zeros(N,1);
for i = 0:N-1
  replika(i+1) = prn(mod(fix(i*Tsa/Tch),1023)+1);
end;

% Analyzed signals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(31)
subplot(311)
len=10000 ;
plot(prn(1:len/100),'.-')
title('Original PRN sequence')
subplot(312)
plot(replika(1:len))
axis([0 len -1.2 1.2])
title('Transmitted PRN signal')
subplot(313)
plot(signal(1:len))
axis([0 len -50 50])
title('Measured received (reflected) signal')


% Looking for the delay
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(32);clf;
CCF=xcorr(replika, signal, 'biased');    % (Received) signal is delayed to (transmitted) replika
% CCF=xcorr(signal, replika,'biased');   % The case of oposite signal order
plot((-N+1:N-1)*Tsa*1e6, CCF );
title('Delay estimation using CCF (Cross-Correlation Function');

CCFsel=CCF(N:end);
[maxval,idx]=max(CCFsel);
delay=(idx)./fsa ;   
disp(['Delay in microseconds : ' num2str(delay*1e6)])

c=3e8 ;  % Speed of the light
distance=delay*c ;
disp(['Measured distance in kilometers : ' num2str(distance/1e3)])





