% B2M31CZS - Estimation of spectral characteristics for random and
% non-stationary signals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% PART1
% Modelling of signals

fs=200;
tmax=40 ;

f1=14;
A1=0.5 ;

f2=27.5 ;
A2=0.4 ;


tt=0:1/fs:tmax-1/fs ;

s1=A1*sin(2*pi*f1*tt)+A2*sin(2*pi*f2*tt) ;

figure(1);
subplot(411)
plot(tt,s1) ;
xlim([0 max(tt)])
title('Two sinusoids')

N=length(s1);
b1=randn(1,N);
b2=rand(1,N);

subplot(412)
plot(tt,b1) ;
xlim([0 max(tt)])
title('White noise - Gaussian')

subplot(413)
plot(tt,b2) ;
xlim([0 max(tt)])
title('White noise - uniform')

Ps1=mean(s1.^2);
Pb1=mean(b1.^2);

SNR=-3;
k=sqrt(Ps1/Pb1*10^(-SNR/10));
x1=s1+k*b1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The 1st CHECKED RESULT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subplot(414)
plot(tt,b1) ;
xlim([0 max(tt)])
title('Two sinusoids in noise')


% PART1
% PSD estimation

wlen=512;

frame1=x1(1:wlen);[psdx1,ff1]=pwelch(x1,wn,[],[],fs,'power');
[psdx2,ff1]=pwelch(x1(1:wlen),wn,[],[],fs,'power');

frame2=x1(wlen+1:2*wlen);

wn=hamming(wlen);
% wn=rectwin(wlen);
wr=rectwin(wlen);
wnor=mean(wr.^2)./mean(wn.^2);  % Normalization factor for spectrum computation

frame1=frame1'.*wn;
frame2=frame2'.*wn;

figure(2)
subplot(211)
plot(frame1);
title('x1 - the 1st short-time frame')
grid

subplot(212)
plot(frame2);
title('x1 - the 2nd short-time frame')
grid

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figures 2 : the 1st checked result
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of power spectrum in dBs (taking into account fs)

Sf1abs=10*log10(wnor/fs/wlen*abs(fft(frame1)).^2);
Sf2abs=10*log10(wnor/fs/wlen*abs(fft(frame2)).^2);
% Sf1abs=10*log10((abs(fft(frame1)).^2)./wlen./fs);
% Sf2abs=10*log10((abs(fft(frame2)).^2)./wlen./fs);

figure(3)[psdx1,ff1]=pwelch(x1,wn,[],[],fs,'power');
[psdx2,ff1]=pwelch(x1(1:wlen),wn,[],[],fs,'power');

ff=(0:wlen/2-1)*fs/wlen;
subplot(211)
plot(ff,Sf1abs(1:wlen/2));
title('x1 - Power spectrum the 1st frame')
grid

subplot(212)
plot(ff,Sf2abs(1:wlen/2));
title('x1 - Power spectrum the 2nd frame')
grid


figure(4)
% pwelch(x1,wn,[],[],fs,'power');
pwelch(x1,wn,[],[],fs);
title('Welch Power spectrum estimate - SMOOTHED')


figure(5)
% pwelch(x1(1:wlen),wn,[],[],fs,'power');
pwelch(x1(1:wlen),wn,[],[],fs);
title('Welch Power spectrum estimate - 1st FRAME ONLY')


figure(6)
[psdx1,ff1]=pwelch(x1,wn,[],[],fs,'power');
[psdx2,ff1]=pwelch(x1(1:wlen),wn,[],[],fs,'power');
% [psdx1,ff1]=pwelch(x1,wn,[],[],fs);
% [psdx2,ff1]=pwelch(x1(1:wlen),wn,[],[],fs);

plot(ff1,Sf1abs(1:wlen/2+1))
hold on
plot(ff1,10*log10(psdx1),'r')
plot(ff1,10*log10(psdx2),'g')
hold off
grid
title('x1 - Power spec. estimate: smoothed vs. unsmoothed')



load e1.mat;
fs=200;

figure(7)

Sf1abs=10*log10(wnor/fs/wlen*abs(fft(e1(1:wlen).*wn)).^2);
[psdx1,ff1]=pwelch(e1,wn,[],[],fs,'power');
[psdx2,ff1]=pwelch(e1(1:wlen),wn,[],[],fs,'power');

plot(ff1,Sf1abs(1:wlen/2+1))
hold on
plot(ff1,10*log10(psdx1),'r')
plot(ff1,10*log10(psdx2),'g')
hold off
grid
title('EEG - Power spec. estimate: smoothed vs. unsmoothed')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figures 6-7 : the 2nd checked result
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of spectrograms for stationary and non-stationary signals

figure(10)
spectrogram(x1,wlen,[],[],fs) ;
title('x1 - spectrogram with frequency at x-axis')

figure(11)
spectrogram(x1,wlen,[],[],fs,'yaxis') ;
title('x1 - spectrogram with frequency at y-axis')

figure(12)
spectrogram(x1,wlen,[],[],fs,'yaxis') ;
colormap jet; colorbar off
title('x1 - spectrogram: other color map, colorbar off')



r1=loadbin('sm2.bin');
% r1=loadbin('sf2.bin');
% r1=loadbin('sm1.bin');
% r1=loadbin('sf1.bin');


fs=16000;
wlen=floor(fs*0.032) ;

figure(13)
subplot(211)
tt=(1:length(r1))./fs ;
plot(tt,r1);
xlim([0 max(tt)])
subplot(212)
spectrogram(r1,wlen,[],[],fs,'yaxis') ;
title(['Speech spectrogram - wlen =' num2str(wlen)])
colorbar off; colormap jet;


figure(14)

subplot(311)
wlen=floor(fs*0.032) ;
spectrogram(r1,wlen,[],[],fs,'yaxis') ;
title(['Speech spectrogram - wlen =' num2str(wlen)])
colorbar off; colormap jet;

subplot(312)
wlen=floor(fs*0.004) ;
spectrogram(r1,wlen,[],wlen,fs,'yaxis') ;
title(['Speech spectrogram - wlen =' num2str(wlen)])
colorbar off; colormap jet;

subplot(313)
wlen=floor(fs*0.256) ;
spectrogram(r1,wlen,[],[],fs,'yaxis') ;
title(['Speech spectrogram - wlen =' num2str(wlen)])
colorbar off; colormap jet;

figure(15)

wlen=floor(fs*0.032) ;

subplot(211)
spectrogram(r1,wlen,[],[],fs,'yaxis') ;
title(['Speech spectrogram - wlen =' num2str(wlen)])
colorbar off; colormap jet;

subplot(212)
pwelch(r1,wlen,[],[],fs) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figures 13-15 : the 3rd checked result
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(16)

% Try also other simulated signals
sig=x1;
% sig=s1;
% sig=e1;
% sig=b1;
% sig=b2;

wlen=512 ;
fs=200;

subplot(211)
spectrogram(sig,wlen,[],[],fs,'yaxis') ;
title(['Spectrogram - wlen =' num2str(wlen)])
colorbar off; colormap jet;

subplot(212)
pwelch(sig,wlen,[],[],fs) ;




% Confidence interval computation


SNR=-20;
k=sqrt(Ps1/Pb1*exp(-SNR/10));
x1=s1+k*b1;
wlen=512 ;

figure(20)
pwelch(x1,wlen,[],[],fs,'ConfidenceLevel',0.98) ;


SNR=0;
k=sqrt(Ps1/Pb1*exp(-SNR/10));
x1=s1+k*b1;
wlen=512 ;

figure(21)
pwelch(x1,wlen,[],[],fs,'ConfidenceLevel',0.98) ;


SNR=20;
k=sqrt(Ps1/Pb1*exp(-SNR/10));
x1=s1+k*b1;
wlen=512 ;

figure(22)
pwelch(x1,wlen,[],[],fs,'ConfidenceLevel',0.98) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figures 20-22 : the 3rd checked result
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Homework

figure(30)

% Try also other simulated signals
sig=s1;


wlen=512 ;
fs=200;

subplot(231)

title(['Spectrogram - wlen =' num2str(wlen)])
spectrogram(sig,wlen,[],[],fs,'yaxis') ;
colorbar off; colormap jet;
view

subplot(232)
spectrogram(sig,wlen,[],[],fs,'yaxis') ;
view(100,40)
colorbar off; colormap jet;

subplot(233)
spectrogram(sig,wlen,[],[],fs,'yaxis') ;
view(40,65)
colorbar off; colormap jet;

subplot(234)

fs=16000;
title(['Spectrogram - wlen =' num2str(wlen)])
spectrogram(r1,wlen,[],[],fs,'yaxis') ;
colorbar off; colormap jet;


subplot(235)


spectrogram(r1,wlen,[],[],fs,'yaxis') ;
colorbar off; colormap jet;
view(100,40)

subplot(236)


spectrogram(r1,wlen,[],[],fs,'yaxis') ;
colorbar off; colormap jet;
view(40,65)



