% Filtering in the frequency domain - cyclic convolution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f=12;
fs=8000;

% Impulse response of MA filter of the length N1 (order N1-1)
N1=101;
h=ones(N1,1)./N1;

% Impulse response of the filter realizing the exponential averaging
% h=0.02*0.98.^(0:N1-1);
% h=h(:);

% Processed signal - low-frequency sine with white noise - length N2
N2=4000;  
ttt=((0:N2-1)./fs)';
x=sin(2*pi*f*ttt)+randn(N2,1) ;

% Computation of linear convolution
ylin=conv(h,x) ;

figure(1)
subplot(311)
stem(h)
title('Impulse response - h[n]')
subplot(312)
plot(x)
title('Input signal - x[n]')
subplot(313)
plot(ylin)
title('Linear convolution')




NDFT=N1+50;   % Volit N = 200, 300, 400, 512

Y=fft(h,NDFT).*fft(x,NDFT) ;
yc=real(ifft(Y,NDFT));

figure(2)
if NDFT<N2, 
    plot(ylin(1:NDFT));
else
    plot(ylin)
end
hold on
plot(yc,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ', RED cyclic, BLUE linear the first ' num2str(NDFT) ' samples'])



% Frame processing without overlapping

NDFT=512;

slen=N2;
wlen=NDFT;

wnum=floor(slen/wlen);

ycseg=[];
for i=1:wnum,
    
    ii=(i-1)*wlen+1;
    jj=(i-1)*wlen+wlen;
   
    frame=x(ii:jj);
   
    Y=fft(h,NDFT).*fft(frame,NDFT) ;
    yframe=real(ifft(Y,NDFT));
   
    ycseg=[ycseg; yframe] ;    
    
end

figure(3)
plot(ylin)
hold on
plot(ycseg,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - no overlap, RED cyclic, BLUE linear'])

% Frame processing OLS

wlen=NDFT;
overlap=N1;
wstep=wlen-overlap;
slen=N2;

wnum2=floor((slen-wlen)/wstep+1);

ycseg=[];
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=x(ii:jj);
   
    Y=fft(h,NDFT).*fft(frame,NDFT) ;
    yframe=real(ifft(Y,NDFT));
   
    if i>1,
        ycseg=[ycseg; yframe(overlap+1:end)];
    else
        ycseg=[ycseg; yframe];
    end
      
    
end

figure(4)
plot(ylin)
hold on
plot(ycseg,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - OLS, RED cyclic, BLUE linear'])

% Frame processing OLA

slen=N2;
overlap=N1-1;
wlen=NDFT-overlap;

wnum3=floor(slen/wlen);

ycseg=zeros(slen+overlap,1);
for i=1:wnum3,
    
    ii=(i-1)*wlen+1;
    jj=(i-1)*wlen+wlen;
   
    frame=[x(ii:jj); zeros(overlap,1)];
   
    Y=fft(h,NDFT).*fft(frame,NDFT) ;
    yframe=real(ifft(Y,NDFT));
   
    ycseg(ii:jj+overlap)= ycseg(ii:jj+overlap)+yframe;
    
end
      

figure(5)
plot(ylin)
hold on
plot(ycseg,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - OLA, RED cyclic, BLUE linear'])



