% FIR Filter Design - Window method

%% 1st checked result

M=30;  % choose also the value 10, 50, 80, 

N=1000;  % choose also a lower value, e.g. 500, 100
Hideal = [ones(1,N/4+1) zeros(1,N/4-1)  zeros(1,N/4)  ones(1,N/4) ];

figure(1); clf
fff=(0:N-1)/N*2;
plot(fff, Hideal, '.-')
ylabel('Magnitude frequency response')
xlabel('Normalized frequency')
title('BLUE - ideal')


drawnow; pause(2)

hideal=fftshift(ifft(Hideal));

figure(2); clf
subplot(211)
nn=0:N-1;
plot(nn,hideal);
ylabel('Impulse response')
xlabel('Discrete time')
title('BLUE - ideal')

drawnow; pause(2)

wn=rectwin(M+1)';
wn=hamming(M+1)'; % wn=wn(1:end-1)

vyber=N/2-M/2+1:N/2+M/2+1;

hfirfs=(0.5*sinc(0.5*(-M/2:M/2))).*wn;  % ATTENTION - sinc(x)=sin(pi*x)/(pi*x)

hfir=hideal(vyber).*wn;

figure(2)
subplot(211)
hold on
plot(vyber-1,hfir,'r');
hold off
title('RED - FIR manual IFFT')
% drawnow; pause(2)

subplot(212)
plot(0:M,hfir,'r');
ylabel('Impulse response')
xlabel('Discrete time')
title('Shortened imp. response')

drawnow; pause(2)


Hfirfs=abs(fft(hfirfs,N));
Hfir=abs(fft(hfir,N));

figure(1)
hold on
plot(fff, Hfir, '-r')
plot(fff, Hfirfs, '-c')
hold off
title('RED - FIR manual IFFT, CYAN - FIR manual FS')
drawnow; pause(2)

hfir1=fir1(M,0.5);
hfir2=fir2(M,[0 0.5 0.5 1],[1 1 0 0])

figure(2)
subplot(212)
hold on
plot(0:M,hfirfs,'c');
plot(0:M,hfir1,'g');
plot(0:M,hfir2,'m');
hold off
legend('FIR manual IFFT', 'FIR manual FS', 'FIR1', 'FIR2','Location','NorthEast')
drawnow; pause(2)

Hfir1=abs(fft(hfir1,N));
Hfir2=abs(fft(hfir2,N));

figure(1)
hold on
plot(fff, Hfir1, '-g')
plot(fff, Hfir2, '-m')
hold off
title('')
legend('ideal', 'FIR manual IFFT', 'FIR manual FS', 'FIR1', 'FIR2','Location','East')
drawnow; pause(2)

figure(3)
plot(fff(1:N/2), 20*log10(Hideal(1:N/2)), '-b')
hold on
plot(fff(1:N/2), 20*log10(Hfir(1:N/2)), '-r')
plot(fff(1:N/2), 20*log10(Hfirfs(1:N/2)), '-c')
plot(fff(1:N/2), 20*log10(Hfir1(1:N/2)), '-g')
plot(fff(1:N/2), 20*log10(Hfir2(1:N/2)), '-m')
hold off
grid
ylabel('Square mag.freq. response in dBs')
xlabel('Normalized frequency')
title('')
legend('ideal', 'FIR manual IFFT', 'FIR manual FS', 'FIR1', 'FIR2','Location','SouthWest')

drawnow; pause(2)

%%  2nd checked result

M=30; % Use also M = 10, 50, 100, 200, 500
Wc=0.5;

b1=fir1(M,Wc) ;
b2=fir1(M,Wc,'high') ;

figure(11)
freqz(b1,1,1000)
title('Low-pass freq. resp.')

figure(12)
freqz(b2,1,1000)
title('High-pass freq. resp.')

figure(13)
subplot(211)
stem(b1)
title('Low-pass impulse resp.')
subplot(212)
stem(b2)
title('High-pass impulse resp.')

%% 3rd checked result (figures 11+12 + others below)

no=randn(20000,1);

nc1=filter(b1,1,no);
nc2=filter(b2,1,no);

figure(14)
subplot(311)
plot(no(1:1000))
title('Original Gaussian white noise')
subplot(312)
plot(nc1(1:1000))
title('Low-band colour noise')
subplot(313)
plot(nc2(1:1000))
title('High-band colour noise')


figure(15)
pwelch(nc1,512)
title('PDS - low-band colour noise')


figure(16)
pwelch(nc2,512)
title('PDS - high-band colour noise')



%% 4th checked result (figures 11+12 + others below)

fc1=300;
fc2=3600;

fs1=8000;
bp1=fir1(100,[fc1/fs1*2 fc2/fs1*2]); % OK is approx for M >= 100

fs2=16000;
bp2=fir1(200,[fc1/fs2*2 fc2/fs2*2]); % OK is approx for M >= 200

fs3=44100;
bp3=fir1(400,[fc1/fs3*2 fc2/fs3*2]); % OK is approx for M >= 400


figure(21)
freqz(bp1,1,1000,fs1) ;
title('Telephone-band-pass filter - fs=8kHz')

figure(22)
freqz(bp2,1,1000,fs2) ;
title('Telephone-band-pass filter - fs=16kHz')

figure(23)
freqz(bp3,1,1000,fs3) ;
title('Telephone-band-pass filter - fs=44.1kHz')

s1=loadbin('s0001.bin');
s2=loadbin('SA106S06.CS0');
s3=loadbin('mc91bc111011.ils_a');
s3=loadbin('fc44be305265.ils_a');

figure(23)
subplot(311)
spectrogram(s1,floor(fs1*0.032),[],[],fs1,'yaxis'); colorbar off; colormap jet;
title('Full-band speech - fs=8kHz')
subplot(312)
spectrogram(s2,floor(fs2*0.032),[],[],fs2,'yaxis'); colorbar off; colormap jet;
title('Full-band speech - fs=16kHz')
subplot(313)
spectrogram(s3,floor(fs3*0.032),[],[],fs3,'yaxis'); colorbar off; colormap jet;
title('Full-band speech - fs=44.1kHz')

st1=filter(bp1,1,s1);
st2=filter(bp2,1,s2);
st3=filter(bp3,1,s3);


figure(24)
subplot(311)
spectrogram(st1,floor(fs1*0.032),[],[],fs1,'yaxis'); colorbar off; colormap jet;
title('Telephone-band speech - fs=8kHz')
subplot(312)
spectrogram(st2,floor(fs2*0.032),[],[],fs2,'yaxis'); colorbar off; colormap jet;  % ZOOM this spectrogram to band 0-4000 Hz
title('Telephone-band speech - fs=16kHz')
subplot(313)
spectrogram(st3,floor(fs3*0.032),[],[],fs3,'yaxis'); colorbar off; colormap jet;  % ZOOM this spectrogram to band 0-4000 Hz
title('Telephone-band speech - fs=44.1kHz')









