% Filtry optimum design

% Distortion

% Rectangular pulses
sig=[];
for i=1:10,
   sig=[sig; ones(100,1)]; 
   sig=[sig; -ones(100,1)]; 
end

% Sample of speech signal
% sig=loadbin('../signaly/vm4.bin')

figure(1)
subplot(211)
plot(sig)
title('Input rectangular signal')

M=100;
b=fir1(M,0.2,'high');

sig1=filter(b,1,sig);

figure(1)
subplot(212)
plot(sig1)
title('High-band signal')


[b1,a1]=cheby1(6,0.05,0.2,'high');

sig2fir=filter(b,1,sig1);
sig2iir=filter(b1,a1,sig1);

figure(2)
subplot(211)
plot(sig1,'r')
hold on
plot(sig2fir(M/2+1:end))
hold off
subplot(212)
plot(sig1,'r')
hold on
plot(sig2iir)
hold off


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Design of IIR filters from tolerance scheme
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fs=8000;
%fs=16000;
fs=44100;


fp1=400;
fp2=3300;
fs1=200;
fs2=3500;
Wp=[fp1, fp2]/fs*2;
Ws=[fs1, fs2]/fs*2;

Rp=1;
Rs=40;

[N1,Wc1]=buttord(Wp,Ws,Rp,Rs);
[b1,a1]=butter(N1,Wc1);

[N2,Wc2]=cheb1ord(Wp,Ws,Rp,Rs);
[b2,a2]=cheby1(N2,Rp,Wc2);

[N3,Wc3]=cheb2ord(Wp,Ws,Rp,Rs);
[b3,a3]=cheby2(N3,Rs,Wc3);

[N4,Wc4]=ellipord(Wp,Ws,Rp,Rs);
[b4,a4]=ellip(N4,Rp,Rs,Wc4);

figure(11);
freqz(b1,a1,1000,fs)

figure(12);
freqz(b2,a2,1000,fs)

figure(13);
freqz(b3,a3,1000,fs)


figure(14);
freqz(b4,a4,1000,fs)

figure(15)
subplot(221)
zplane(b1,a1);
subplot(222)
zplane(b2,a2);
subplot(223)
zplane(b3,a3);
subplot(224)
zplane(b4,a4);

figure(16)
subplot(221)
impz(b1,a1);
subplot(222)
impz(b2,a2);
subplot(223)
impz(b3,a3);
subplot(224)
impz(b4,a4);


%% Filtering 

load speech_8_16_44.mat ;

if fs==8000,
    sigin=sig8;
    % All designed filters for 8kHz are stable and all of them can be used
    aa=a1;
    bb=b1;
elseif fs==16000,
    sigin=sig16;
    aa=a4;
    bb=b4;
elseif fs==44100,
    sigin=sig44;
    % Filtering using IIR filter is currently not possible - all filters for 44.1kHz are not stable
    % Stable solution for this case must be found after some change of input
    % requirements
end

if fs==8000 || fs==16000
    
    sigout=filter(bb,aa,sigin);

    figure(17)
    subplot(211)
    spectrogram(sigin,floor(0.032*fs),[],[],fs,'yaxis')
    colorbar off; colormap jet;
    subplot(212)
    spectrogram(sigout,floor(0.032*fs),[],[],fs,'yaxis')
    colorbar off; colormap jet;

end



%% Design of FIR filters from tolerance scheme

devp=1-10^(-Rp/20);
devs=10^(-Rs/20);


fc = [fs1 fp1 fp2 fs2]/fs*2;
amp = [0 1 0];
dev = [devs devp devs];
[Nk, W, beta, ftype] = kaiserord ( fc, amp, dev ) ;
bkaiser = fir1 ( Nk, W, ftype, kaiser(Nk+1, beta) ) ;

figure(21)
freqz(bkaiser,1,1000,fs)
title(['Kaiser-based FIR filter, order is ' num2str(Nk)])

F=[fs1 fp1 fp2 fs2];
[Npm,Fo,Ao,W] = firpmord(F,[0 1 0],[devs devp devs],fs);
bpm = firpm(Npm,Fo,Ao,W);

figure(22)
freqz(bpm,1,1000,fs)
title(['Parks-McClellan optimum FIR filter, order is ' num2str(Npm)])


sigfirkai=filter(bkaiser,1,sigin);
sigfirpm=filter(bpm,1,sigin);
 
figure(23)
subplot(311)
spectrogram(sigin,floor(0.032*fs),[],[],fs,'yaxis')
xlabel(''); ylabel('')
title('FIR filtering: Input / Kaiser FIR / Parks-McClellan FIR')
colorbar off; colormap jet;
subplot(312)
spectrogram(sigfirkai,floor(0.032*fs),[],[],fs,'yaxis')
xlabel(''); ylabel('')
colorbar off; colormap jet;
subplot(313)
spectrogram(sigfirpm,floor(0.032*fs),[],[],fs,'yaxis')
xlabel(''); ylabel('')
colorbar off; colormap jet;

%% Filters for 50Hz removing from ECG signal

% x=load('ekg1.asc');
x=load('ekg50.asc');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fs=200;
fn=[49 51];
Wn=2*fn/fs;

Rp=1;
Rs=40;

type='stop';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Hamming window FIR design 
Nfir=200;
b=fir1(Nfir,Wn,type);


figure(30); freqz(b,1,1000,fs) ;
Hfir=freqz(b,1,1000,fs) ;
title(['Hamming window FIR filter of the order ' num2str(Nfir)])

yfir=filter(b,1,x);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Butterworth - empiric desing

Nbutt=6;
[b1,a1]=butter(Nbutt,Wn,type);

figure(31); freqz(b1,a1,1000,fs) ;
H1=freqz(b1,a1,1000,fs) ;
title(['Butterworth filter of the order ' num2str(Nbutt)])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Inverse Chebyshev - empiric design

Ncheby=6;
[b2,a2]=cheby1(Ncheby,Rp,Wn,type);


figure(32); freqz(b2,a2,1000,fs) ;
H2=freqz(b2,a2,1000,fs) ;
title(['Chebyshev filter of the order ' num2str(Ncheby)])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Inverse Chebyshev - empiric design

Ncheby=6;
[b3,a3]=cheby2(Ncheby,Rs,Wn,type);


figure(33); freqz(b3,a3,1000,fs) ;
H3=freqz(b3,a3,1000,fs) ;
title(['Inverse Chebyshev filter of the order ' num2str(Ncheby)])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Elliptic -empiric design

Nellip=6;
[b4,a4]=ellip(Nellip,Rp,Rs,Wn,type);


figure(34); freqz(b4,a4,1000,fs) ;
[H4,ff]=freqz(b4,a4,1000,fs) ;
title(['Elliptic filter of the order ' num2str(Nellip)])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(41)
plot(ff,20*log10(abs(Hfir)),'b');
hold on
plot(ff,20*log10(abs(H1)),'g');
plot(ff,20*log10(abs(H2)),'r');
plot(ff,20*log10(abs(H3)),'m');
plot(ff,20*log10(abs(H4)),'k');
hold off
axis([0 fs/2 -80 10])
title('Magnitude responses of designed filters')
grid


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(42)
plot(ff,unwrap(angle(Hfir)),'b');
hold on
plot(ff,unwrap(angle(H1)),'g');
plot(ff,unwrap(angle(H2)),'r');
plot(ff,unwrap(angle(H3)),'m');
plot(ff,unwrap(angle(H4)),'k');
hold off
title('Phase responses of designed filters')
%axis([0 fs/2 -80 10])
grid


figure(42)
subplot(221)
zplane(b1,a1);
subplot(222)
zplane(b2,a2);
subplot(223)
zplane(b3,a3);
subplot(224)
zplane(b4,a4);

figure(43)
subplot(221)
impz(b1,a1);
subplot(222)
impz(b2,a2);
subplot(223)
impz(b3,a3);
subplot(224)
impz(b4,a4);


